from utils.application import app
from routes.category import get_category
from routes.home import get_home
from routes.order import post_create_order, pay_order, get_all_orders
from routes.login import post_login, post_logout, post_register
from routes.product import get_product_by_id, get_product_all, get_products_by_search

# Routing

# Routing for homepage
@app.route('/', methods=['GET'])
def home():
    return get_home()

# Routes for getting the different categories
@app.route('/category/', methods=['GET'])
@app.route('/category/<string:category_name>', methods=['GET'])
def category(category_name=None):
    return get_category(category_name)

# Route for creating the order
@app.route('/order/', methods=['POST'])
def create_order():
    return post_create_order()

@app.route('/order/', methods=['GET'])
def get_order():
    return get_all_orders()

# Route for paying an order
@app.route('/order/payment/', methods=['POST'])
def pay():
    return pay_order()

# Route for logging out of the app
@app.route('/logout/', methods=['POST'])
def logout():
    return post_logout()

# Route for logging in
@app.route('/login/', methods=['POST'])
def login():
    return post_login()

# Route for registering a new user
@app.route('/register/', methods=['POST'])
def register():
    return post_register()

# Route for one specific product, or product page
@app.route('/product/<int:product_id>', methods=['GET'])
def get_product(product_id):
    return get_product_by_id(product_id)

# Route for getting all products
@app.route('/product/', methods=['GET'])
def get_products():
    return get_product_all()

# Route for searching for a new product
@app.route('/product/search/<string:search>', methods=['GET'])
def search_products(search):
    return get_products_by_search(search)

if __name__ == '__main__':
    app.run(debug=True, port=8080)