from utils.application import mysql
from flask import jsonify
from utils.helpers import sql_product_to_json

def get_category(category_name=None):
    # If category_name is not None, get one category by name
    if (category_name is not None):
        return get_category_by_name(category_name)
    else:
        # Construct the query to get the category
        cur = mysql.connection.cursor()
        cur.execute('''
            SELECT category.name, category.description, sub_category.name, sub_category.description FROM category
            LEFT JOIN sub_category_in_category ON sub_category_in_category.parent_category_id = category.category_id
            LEFT JOIN category AS sub_category ON sub_category.category_id = sub_category_in_category.sub_category_id
        ''')

        data = cur.fetchall()
        cur.close()

        # If no data is found, return an empty list
        categories = []
        for row in data:
            #Extract data from row
            categoryName, categoryDescription, subCategoryName, subCategoryDescription = row
            # Check if category exists already in category
            category = next((x for x in categories if x["categoryName"] == categoryName), None)

            # If it does not exist, create it
            if category is None:
                category = {
                    "categoryName": categoryName,
                    "categoryDescription": categoryDescription,
                    "subCategories": []
                }
                categories.append(category)

            #Always append subcategories extracted from row
            if (subCategoryName is not None):
                category["subCategories"].append({
                    "subCategoryName": subCategoryName,
                    "subCategoryDescription": subCategoryDescription
                })

        return jsonify(categories)


def get_category_by_name(category_name):
    # Get the category by a name
    cur = mysql.connection.cursor()
    cur.execute('''
        SELECT category.name, category.description, sub_category.name, sub_category.description FROM category
        LEFT JOIN sub_category_in_category ON sub_category_in_category.parent_category_id = category.category_id
        LEFT JOIN category AS sub_category ON sub_category.category_id = sub_category_in_category.sub_category_id
        WHERE category.name = %s
    ''', (category_name,))

    categoryData = cur.fetchall()

    # Get all of the products for that one category.
    cur.execute('''
        SELECT product.product_id, product.name, product.description, product.price, product.stock_quantity FROM product
        INNER JOIN category ON category.category_id = product.category_id
        WHERE category.name = %s
    ''', (category_name,))
    productData = cur.fetchall()
    cur.close()

    # Make sure we have a category
    if len(categoryData) == 0:
        return jsonify({"error": "Category not found"}), 404

    # Extract data from first row to create category object
    categoryName, categoryDescription, subCategoryName, subCategoryDescription = categoryData[0]
    category = {
        "categoryName": categoryName,
        "categoryDescription": categoryDescription,
        "subCategories": [],
        "products": []
    }
    for row in categoryData:
        #Extract data from row to create subcategory object
        categoryName, categoryDescription, subCategoryName, subCategoryDescription = row
        if (subCategoryName is not None):
            category["subCategories"].append({
                "subCategoryName": subCategoryName,
                "subCategoryDescription": subCategoryDescription
            })


    for row in productData:
        # Extract product-info for each product
        productId, productName, productDescription, productPrice, productStockQuantity = row
        category["products"].append({
            "productId": productId,
            "productName": productName,
            "productDescription": productDescription,
            "productPrice": productPrice,
            "productStockQuantity": productStockQuantity
        })

    return jsonify(category)