from utils.application import mysql
from flask import request, jsonify, make_response
import bcrypt

def post_logout():
    userId = request.cookies.get('logged_in')
    if userId is None:
        return jsonify({"message": "No user logged in"}), 400

    response = make_response(jsonify({"message": "Logout successful"}))
    # Cookie expires immediately
    response.set_cookie('logged_in', '', expires=0)
    return response

def post_login():
    # Make sure we are not already logged in to another user
    userId = request.cookies.get('logged_in')
    if userId is not None:
        return jsonify({"message": "A user is already logged in"}), 400

    # Get data from request
    data = request.json
    email = data["email"]
    password = data["password"]

    # Check if user exists and/or password exists
    cur = mysql.connection.cursor()
    cur.execute('''SELECT user.user_id, user.email, user.hash, user.salt FROM user WHERE user.email = %s''', (email,))
    dbData = cur.fetchall()
    cur.close()

    if len(dbData) <= 0:
        return jsonify({"message": "Incorrect login information."}), 400

    # Deconstruct dbData[0]
    userId, email, hash, salt = dbData[0]

    # Check if user exists
    if len(dbData) <= 0:
        return jsonify({"message": "Incorrect login information."}), 400

    # Retrieve the password and hash the password to check with DB hash
    salt = salt.encode("utf-8")
    new_hash = hash_function(password, salt)

    # Password check
    if new_hash.decode("utf-8") != hash:
        return jsonify({"message": "Incorrect login information."}), 400

    # Set cookie and make it last 24 hours
    response = make_response(jsonify({"message": "Login successful"}))
    userId = str(userId)
    response.set_cookie('logged_in', userId, max_age=60*60*24)
    return response, 200

def post_register():
    # Make sure we are not already logged in to another user
    userId = request.cookies.get('logged_in')
    if userId is not None:
        return jsonify({"message": "A user is already logged in"}), 400

    # Get data from request
    data = request.json
    email = data['email']
    password = data['password']
    firstname = data['firstname']
    lastname = data['lastname']
    address = data['address']

    # Hash and salt password
    salt = bcrypt.gensalt()
    password = hash_function(password, salt)

    # Check if email already exists
    cur = mysql.connection.cursor()
    cur.execute('''SELECT user.email FROM user WHERE user.email = %s''', (email,))
    data = cur.fetchall()
    cur.close()

    # If email already exists, return error
    if len(data) > 0:
        return jsonify({"message": "Email already exists"}), 400

    # Instert the user information into the DB
    cur = mysql.connection.cursor()
    cur.execute('''INSERT INTO user_details (email, first_name, last_name, address) VALUES (%s, %s, %s, %s)''', (email, firstname, lastname, address))
    cur.execute('''INSERT INTO user (email, hash, salt) VALUES (%s, %s, %s)''', (email, password, salt))
    mysql.connection.commit()
    cur.close()

    return "", 201


def hash_function(password, salt):
    password = password.encode("utf-8")
    # Hash the password and return the hashed value
    hashed = bcrypt.hashpw(password, salt)
    return hashed